#!/usr/bin/python

import json
import re
import shutil
import socket
import subprocess
import sys

def error(msg):
    print json.dumps({
        "failed" : True,
        "msg"    : msg
    })
    sys.exit(1)

if len(sys.argv) != 2:
    error("Need IP address as module argument")

try:
    args_file = sys.argv[1]
    args_line = file(args_file).read().strip("\n")
    args_re = re.compile('\s(?=(?:[^"]*"[^"]*")*[^"]*$)')
    args_arr = re.split(args_re, args_line)
    args = {a.split('=')[0]:a.split('=')[1].replace('"','') for a in args_arr if a.find('=') > 0}
except Exception, e:
    error("Cannot read IP from input file: " + str(e))

# validate input - must be IPv4 adddress
try:
    ip = args['ip_address']
    socket.inet_pton(socket.AF_INET, ip)
except:
    error("Received invalid IPv4 address: " + ip)

host = "rabbit-" + ip.replace(".", "-")
# Check if host can be found by short name
try:
    if ip != socket.gethostbyname(host):
        error(host + " resolves to wrong ip: " + ip)
except socket.gaierror:
    # Add generated name to /etc/hosts
    with open('/etc/hosts') as f:
        old_entries = (line.strip('\n') for line in f.readlines())
    new_entries = []
    need_new_entry = True
    for entry in old_entries:
        if entry.startswith(ip):
            if host in entry.split():
                error("host name not resolvable: " + host)
            else:
                entry += " " + host
                need_new_entry = False
        new_entries.append(entry)
    if need_new_entry:
        new_entries.append(ip + " " + host)

    shutil.copy("/etc/hosts", "/etc/hosts.bak")
    with open("/etc/hosts", "w") as f:
        f.write("\n".join(new_entries) + "\n")

answer = { "rabbitmq_host" : host, "failed" : False }

# If this ip belongs to this host also set ansible facts
ip_entries = subprocess.check_output(["/usr/sbin/ip", "addr"])
pattern = re.compile(" +inet (\d+\.\d+\.\d+\.\d+)")
addresses = re.findall(pattern, ip_entries)
if ip in addresses:
    answer["ansible_facts"] = {
        "rabbitmq_host" : host, "rabbitmq_ip" : ip
    }

print json.dumps(answer)